//============================================
//Test  SL015B3
//PIC C source code
//PIC16F887(8MHz) + PICC 9.70
//============================================
#include <htc.h>
#include <pic16f887.h>
#include <string.h>

__CONFIG(DEBUGEN&LVPDIS&FCMEN&IESOEN&BOREN&UNPROTECT&DUNPROTECT&MCLREN&PWRTEN&WDTDIS&INTIO);

//============================================
//  PIN define
//============================================
#define RX_PIN TRISC7
#define TX_PIN TRISC6
#define Buzzer RB2
#define CARDIN_SL015M3	RC0

//============================================
//  Uart status words define
//============================================
#define UARTSTATUS_FREE 0
#define UARTSTATUS_TX 1
#define UARTSTATUS_TXSUCC 2
#define UARTSTATUS_RXSUCC 3
#define UARTSTATUS_RXERR 4

//============================================
//  Card type define
//============================================
#define CARDTYPE_S50 0
#define CARDTYPE_S70 1
#define CARDTYPE_ProX 2
#define CARDTYPE_Pro 3
#define CARDTYPE_UL 4
#define CARDTYPE_DES 5
#define CARDTYPE_TAG 6
#define CARDTYPE_ICODE 7

//============================================
//  global variable define
//============================================
bank1 unsigned int g_iTimer;
bank1 unsigned char g_cTimer_Buzzer;
unsigned char g_bOverTime;
bit g_bUartTxHead;
bit g_bUartRxHead;
const unsigned char *g_pUartTxDat;
unsigned char g_cUartTxCheckSum;
unsigned char g_cUartRxCnt;
unsigned char g_cUartStatus;
bank1 unsigned char g_cRxBuf[40];
bank1 unsigned char g_cUartTxCnt;
bank1 unsigned char g_cUartTxDataLen;
bank1 unsigned char g_cCardType;

//============================================
//  Command List, preamble + length + command 
//============================================
const unsigned char GetCardInformation[3]={0xBA,0x02,0x31};       
const unsigned char GetBlcok0SecurityStatus[5]={0xBA,0x04,0x32,0x0,0x01};
const unsigned char ReadBlocks[5]={0xBA,0x04,0x33,0x0,0x01};                                     
const unsigned char WriteBlock0[8]={0xBA,0x07,0x34,0x0,0x11,0x22,0x33,0x44};                                
const unsigned char WriteAFI[4]={0xBA,0x03,0x35,0xAA};  
const unsigned char WriteDSFID[4]={0xBA,0x03,0x36,0xBB};                                
const unsigned char LockBlock0[4]={0xBA,0x03,0x37,0x0};                                
const unsigned char LockAFI[3]={0xBA,0x02,0x38};                                          
const unsigned char LockDSFID[3]={0xBA,0x02,0x39};    
const unsigned char TurnOnRed[5]={0xBA,0x03,0x40,0x01,0x00}; 
const unsigned char TurnOffRed[5]={0xBA,0x03,0x40,0x01,0x01};             

//============================================
//  procedure define
//============================================
void Init_Hardware(void);
void BuzzerOn(void);
void Start_Time(unsigned int ms);
void SendBuf_UART(const unsigned char *dat,unsigned char len);
void Test_SL015M3(void);

//============================================
//  initialize system hardware config
//  timer,uart,port
//============================================
void Init_Hardware(void)
{	
  OSCCON=0x71;		//8M
  while(HTS==0);		//ȴȶ
  ANSEL=0x00;
  ANSELH=0x00;  
  TRISB2=0;
  OPTION=0x07;
  TMR0=0xb1;
  T0IE=1;
  RX_PIN=1;
  TX_PIN=1;
  Buzzer=0;
  TXSTA=0x04;
  RCSTA=0x90;
  BAUDCTL=0x08;
  SPBRG=0x10;
  SPBRGH=0;
  RCIE=1;
  TXEN=1;
  PEIE=1;
  GIE=1;
}

//============================================
//  interrupt service
//  timer interrupt
//  uart rx interrupt
//============================================
void interrupt isr(void)
{
  unsigned char dat;
  unsigned char CheckSum;
  unsigned char counter;
  if(T0IF)
  {
    T0IF=0;
    TMR0=0xb1;
    if(g_iTimer!=0)
    {
      g_iTimer--;
      if(g_iTimer==0) g_bOverTime=1;
    }
    if(g_cTimer_Buzzer!=0)
    {
      g_cTimer_Buzzer--;
      if(g_cTimer_Buzzer==0) Buzzer=0;
    }	
  }
  if(RCIF)
  {
    dat=RCREG;
    {
      g_cRxBuf[g_cUartRxCnt]=dat;
      g_cUartRxCnt++;
      if(g_cUartRxCnt==g_cRxBuf[1]+2)
      {
        CheckSum=0;
        for(counter=0;counter<g_cUartRxCnt;counter++) CheckSum=CheckSum^g_cRxBuf[counter];
        if(CheckSum==0) g_cUartStatus=UARTSTATUS_RXSUCC;
        else g_cUartStatus=UARTSTATUS_RXERR;
      }
    }
  }
}

//============================================
//  trun on buzzer about 100mS
//============================================
void BuzzerOn(void)
{
  Buzzer=1;
  g_cTimer_Buzzer=10;
}

//============================================
//  start timer
//============================================
void Start_Time(unsigned int ms)
{
  g_iTimer=ms;
  g_bOverTime=0;
}

//============================================
//  send buffer by uart
//============================================
void SendBuf_UART(const unsigned char *dat,unsigned char len)
{
  unsigned char tmp;
  g_cUartStatus=UARTSTATUS_TX;
  g_pUartTxDat=dat;
  g_cUartTxDataLen=len;
  g_cUartTxCnt=0;
  g_cUartTxCheckSum=0;
  g_bUartTxHead=0;
  while(g_cUartTxCnt!=g_cUartTxDataLen)
  {
    tmp=g_pUartTxDat[g_cUartTxCnt];
    g_cUartTxCheckSum=g_cUartTxCheckSum^tmp;
    g_cUartTxCnt++;
    while(TXIF==0);
    TXREG=tmp;
  }
  g_cUartStatus=UARTSTATUS_TXSUCC;
  g_cUartRxCnt=0;
  while(TXIF==0);
  TXREG=g_cUartTxCheckSum;
}

#define RCVCMD_SL015M3 g_cRxBuf[2]
#define RCVSTA_SL015M3 g_cRxBuf[3]

void Test_SL015M3(void)
{
  g_cCardType=0xff;
  if(CARDIN_SL015M3) return;
  SendBuf_UART(GetCardInformation,sizeof(GetCardInformation));	
  Start_Time(30);
  while((g_cUartStatus!=UARTSTATUS_RXERR)&&(g_cUartStatus!=UARTSTATUS_RXSUCC)&&(g_bOverTime==0));
  if((g_cUartStatus!=UARTSTATUS_RXSUCC)||(RCVCMD_SL015M3!=0x31)||(RCVSTA_SL015M3!=0)) return;
  if(g_cRxBuf[14]==0x31) g_cCardType=CARDTYPE_TAG;
  else if(g_cRxBuf[14]==0x32) g_cCardType=CARDTYPE_ICODE;
  else return;
  SendBuf_UART(GetBlcok0SecurityStatus,sizeof(GetBlcok0SecurityStatus));	
  Start_Time(30);
  while((g_cUartStatus!=UARTSTATUS_RXERR)&&(g_cUartStatus!=UARTSTATUS_RXSUCC)&&(g_bOverTime==0));
  if((g_cUartStatus!=UARTSTATUS_RXSUCC)||(RCVCMD_SL015M3!=0x32)||(RCVSTA_SL015M3!=0)) return;
  if(g_cRxBuf[4]!=0) return;
  SendBuf_UART(WriteBlock0,sizeof(WriteBlock0));	
  Start_Time(30);
  while((g_cUartStatus!=UARTSTATUS_RXERR)&&(g_cUartStatus!=UARTSTATUS_RXSUCC)&&(g_bOverTime==0));
  if((g_cUartStatus!=UARTSTATUS_RXSUCC)||(RCVCMD_SL015M3!=0x34)||(RCVSTA_SL015M3!=0)) return;
  if(memcmp(&WriteBlock0[4],&g_cRxBuf[4],4)!=0) return;
  SendBuf_UART(ReadBlocks,sizeof(ReadBlocks));	
  Start_Time(30);
  while((g_cUartStatus!=UARTSTATUS_RXERR)&&(g_cUartStatus!=UARTSTATUS_RXSUCC)&&(g_bOverTime==0));
  if((g_cUartStatus!=UARTSTATUS_RXSUCC)||(RCVCMD_SL015M3!=0x33)||(RCVSTA_SL015M3!=0)) return;
  if(memcmp(&WriteBlock0[4],&g_cRxBuf[4],4)!=0) return;
  SendBuf_UART(WriteAFI,sizeof(WriteAFI));	
  Start_Time(30);
  while((g_cUartStatus!=UARTSTATUS_RXERR)&&(g_cUartStatus!=UARTSTATUS_RXSUCC)&&(g_bOverTime==0));
  if((g_cUartStatus!=UARTSTATUS_RXSUCC)||(RCVCMD_SL015M3!=0x35)||(RCVSTA_SL015M3!=0)) return;
  if(memcmp(&WriteAFI[3],&g_cRxBuf[4],1)!=0) return;
  SendBuf_UART(WriteDSFID,sizeof(WriteDSFID));	
  Start_Time(30);
  while((g_cUartStatus!=UARTSTATUS_RXERR)&&(g_cUartStatus!=UARTSTATUS_RXSUCC)&&(g_bOverTime==0));
  if((g_cUartStatus!=UARTSTATUS_RXSUCC)||(RCVCMD_SL015M3!=0x36)||(RCVSTA_SL015M3!=0)) return;
  if(memcmp(&WriteDSFID[3],&g_cRxBuf[4],1)!=0) return;
  BuzzerOn();
  //Glare Red_Led to indicate working ok
  SendBuf_UART(TurnOnRed,sizeof(TurnOnRed));
  Start_Time(30);
  while((g_cUartStatus!=UARTSTATUS_RXERR)&&(g_cUartStatus!=UARTSTATUS_RXSUCC)&&(g_bOverTime==0));
  Start_Time(30);
  while(g_bOverTime==0);
  SendBuf_UART(TurnOffRed,sizeof(TurnOffRed));
  Start_Time(30);
  while((g_cUartStatus!=UARTSTATUS_RXERR)&&(g_cUartStatus!=UARTSTATUS_RXSUCC)&&(g_bOverTime==0));
}

void main(void)
{
  Init_Hardware();
  g_iTimer=0;
  while(1) Test_SL015M3();
}

